// ==============================================================
//						  RocketStageBuilder
//
//						  by Cristian Bontas
//
// RocketStageBuilder.java
//
// ==============================================================

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Properties;

public class RocketStageBuilder {
	
	public final static String PARAM_STAGE_NAME = "STAGE_NAME";
	
	public final static String PARAM_TANK_SECTIONS = "TANK_SECTIONS";
	public final static String PARAM_TANK_LENGTH = "TANK_LENGTH";
	public final static String PARAM_TANK_RADIUS = "TANK_RADIUS";
	
	public final static String PARAM_NOZZLE_SECTIONS = "NOZZLE_SECTIONS";
	public final static String PARAM_NOZZLE_LENGTH = "PARAM_NOZZLE_LENGTH";
	public final static String PARAM_NOZZLE_THROAT_RADIUS = "NOZZLE_THROAT_RADIUS";
	public final static String PARAM_NOZZLE_EXIT_RADIUS = "NOZZLE_EXIT_RADIUS";

	private static String stageName = "RocketStageBuilder";
	
	public static int tankSections = 12;
	public static double tankLength = 3;
	public static double tankRadius = 0.5;
	
	public static int nozzleSections = 12;
	public static double nozzleLength = 1;
	public static double nozzleThroatRadius = 0.25;
	public static double nozzlaExitRadius = 0.5;
	

	public static void main(String[] args) {
		System.out.println("RocketStageBuilder started!");
		try {
			Properties prop = new Properties();
			InputStream input = null;
			input = new FileInputStream("RocketStageBuilder.properties");
			prop.load(input);
			
			stageName = prop.getProperty(PARAM_STAGE_NAME, stageName);
			try {
				tankSections = Integer.parseInt(prop.getProperty(PARAM_TANK_SECTIONS, new Integer(tankSections).toString()));
			} catch(Throwable th) {
			}
			try {
				tankLength = Double.parseDouble(prop.getProperty(PARAM_TANK_LENGTH, new Double(tankLength).toString()));
			} catch(Throwable th) {
			}
			try {
				tankRadius = Double.parseDouble(prop.getProperty(PARAM_TANK_RADIUS, new Double(tankRadius).toString()));
			} catch(Throwable th) {
			}
			try {
				nozzleSections = Integer.parseInt(prop.getProperty(PARAM_NOZZLE_SECTIONS, new Integer(nozzleSections).toString()));
			} catch(Throwable th) {
			}
			try {
				nozzleLength = Double.parseDouble(prop.getProperty(PARAM_NOZZLE_LENGTH, new Double(nozzleLength).toString()));
			} catch(Throwable th) {
			}
			try {
				nozzleThroatRadius = Double.parseDouble(prop.getProperty(PARAM_NOZZLE_THROAT_RADIUS, new Double(nozzleThroatRadius).toString()));
			} catch(Throwable th) {
			}
			try {
				nozzlaExitRadius = Double.parseDouble(prop.getProperty(PARAM_NOZZLE_EXIT_RADIUS, new Double(nozzlaExitRadius).toString()));
			} catch(Throwable th) {
			}
			
			
			PrintWriter writer = new PrintWriter(stageName + ".msh", "UTF-8");
			writer.println("MSHX1");
			writer.println("GROUPS 2");
			
			// tank mesh
			writer.println("GEOM " + (2 * tankSections + 2) + " " + (4 * tankSections));
			writer.println("0 0 0");
			for (int i=1; i <= tankSections; i++) {
				double x = tankRadius * Math.sin(i * (2 * Math.PI) / tankSections);
				double y = tankRadius * Math.cos(i * (2 * Math.PI) / tankSections);
				double z = 0;
				writer.println(x + " " + y + " " + z);
				z = tankLength;
				writer.println(x + " " + y + " " + z);
			}
			writer.println("0 0 " + tankLength);

			for (int i=1; i < tankSections; i++) {
				writer.println("0 " + (2 * i - 1) + " " + (2 * i + 1));
			}
			writer.println("0 " + (2 * tankSections - 1) + " 1");

			for (int i=1; i < tankSections; i++) {
				writer.println((2*tankSections + 1) + " " + (2 * i + 2) + " " + (2 * i));
			}
			writer.println((2*tankSections + 1) + " 2" + " " + (2 * tankSections));

			for (int i=1; i < tankSections; i++) {
				writer.println((2*i - 1) + " " + (2 * i) + " " + (2 * i + 1));
				writer.println((2*i + 1) + " " + (2 * i) + " " + (2 * i + 2));
			}
			writer.println((2*tankSections - 1) + " " + (2 * tankSections) + " 1");
			writer.println("1 " + (2 * tankSections) + " 2");
			
			// nozzle mesh
			writer.println("GEOM " + (2 * nozzleSections) + " " + (4 * nozzleSections));
			for (int i=1; i <= nozzleSections; i++) {
				double x = nozzlaExitRadius * Math.sin(i * (2 * Math.PI) / nozzleSections);
				double y = nozzlaExitRadius * Math.cos(i * (2 * Math.PI) / nozzleSections);
				double z = -nozzleLength;
				writer.println(x + " " + y + " " + z);
				x = nozzleThroatRadius * Math.sin(i * (2 * Math.PI) / nozzleSections);
				y = nozzleThroatRadius * Math.cos(i * (2 * Math.PI) / nozzleSections);
				z = 0;
				writer.println(x + " " + y + " " + z);
			}

			for (int i=0; i < nozzleSections - 1; i++) {
				writer.println((2*i) + " " + (2 * i + 1) + " " + (2 * i + 2));
				writer.println((2*i) + " " + (2 * i + 2) + " " + (2 * i + 1));

				writer.println((2*i + 2) + " " + (2 * i + 1) + " " + (2 * i + 3));
				writer.println((2*i + 2) + " " + (2 * i + 3) + " " + (2 * i + 1));
			}
			writer.println((2*nozzleSections - 2) + " " + (2 * nozzleSections - 1) + " 0");
			writer.println((2*nozzleSections - 2) + " 0" + " " + (2 * nozzleSections - 1));
			writer.println("0 " + (2 * nozzleSections - 1) + " 1");
			writer.println("0 " + "1 " + (2 * nozzleSections - 1));

			writer.println("MATERIALS 0");
			writer.println("TEXTURES 0");
			writer.close();
		} catch (IOException io) {
			io.printStackTrace();
		} finally {
		}
		System.out.println("RocketStageBuilder ended!");
	}
}
